==> w1druppel-0.1.7.tgz: package/src/search.c <==
#include <stddef.h>

#include <alleycat.h>

#include "private/benchmark.h"
#include "private/ds2482.h"
#include "private/i2c.h"
// --- this is circular, seems ok.
#include "private/search.h"
#include "private/util.h"

#ifdef SEARCH_TYPE_W1DIRECT

static int determine_bit (int const desc_bit, int const bit) {
  int prevDeviceId = 0;
  if      (bit == desc_bit) return 1;
  else if (bit >  desc_bit) return 0;
  else return ((prevDeviceId >> bit) & 0x1);
}

static bool some_device_responded (uint8_t const triplet) {
  return (triplet & 0x03) != 0x03;
}

// --- e.g. for 0x33, the LSB is 00110011, and reversed it's {1, 1, 0, 0, 1, 1, 0, 0}.
static void get_family_reversed (uint8_t const device_family, bool * const family_reversed) {
  for (int i = 0; i < 8; i++) {
    int x = device_family & (1 << i);
    family_reversed[i] = (bool) x;
  }
}

/** `device_family` is an optional hex value to filter for a specific family, which speeds up the
 * search slightly. For DS1961 it's 0x33. Set to 0 to disable filtering.
 */

static bool search_inner (const struct master * const master, uint8_t const device_family, uint64_t * const ret) {
  uint64_t device_id = 0;
  int desc_bit = 64;
  int bit = -1;

  bool family_reversed[8];
  if (device_family) get_family_reversed (device_family, family_reversed);

  bool broke = false;
  // --- each bit ~ 3ms, all I/O
  while (++bit < 64) {
    benchmark ("inner loop, bit %d", bit);
    uint8_t triplet;
    uint8_t search_bit = determine_bit (desc_bit, bit);
    if (bit == 0) debug("search_bit %d\n", search_bit);
    try (ds2482_triplet, (master, search_bit, &triplet))
    benchmark ("inner loop, bit %d, after I/O", bit);

    if (!some_device_responded (triplet)) {
      if (bit != 0) debug ("Quitting at bit %d\n", bit);
      broke = true;
      break;
    }

    uint64_t bit_val = triplet >> 2;
    device_id |= bit_val << bit;

    if (device_family) {
      if (bit < 8 && family_reversed[bit] != (bool) bit_val) {
        debug ("bad read, bit = %d, expected = %d, bit_val = %d\n", bit, family_reversed[bit], bit_val);
        broke = true;
        break;
      }
    }
  }

  *ret = broke ? 0 : device_id;

  return true;
}
#endif

#ifdef SEARCH_TYPE_MAXIM
static bool search_inner (const struct master * const master, uint64_t * const ret) {
  benchmark ("search_inner_maxim start");
  *ret = 0;

  unsigned char serial_rom[8] = {0};
  uint64_t serial_rom64 = 0;

  bool search_direction;
  int rom_bit_number = 1;
  int rom_byte_number = 0;
  unsigned char rom_byte_mask = 1;
  int search_result = 0;

  do {
    bool id_bit;
    bool cmp_id_bit;
    try (ds2482_read_bit, (master, &id_bit))
    try (ds2482_read_bit, (master, &cmp_id_bit))

    if (id_bit && cmp_id_bit) break;
    if (id_bit != cmp_id_bit) search_direction = id_bit;
    else search_direction = false;

    if (search_direction) serial_rom[rom_byte_number] |= rom_byte_mask;
    else serial_rom[rom_byte_number] &= ~rom_byte_mask;

    try (ds2482_write_bit, (master, (search_direction)))

    rom_bit_number++;
    rom_byte_mask <<= 1;

    serial_rom64 |= ((uint64_t) serial_rom[rom_byte_number]) << (rom_byte_number * 8);

    if (rom_byte_mask == 0) {
      rom_byte_number++;
      rom_byte_mask = 1;
    }
  } while (rom_byte_number < 8);

  if (rom_bit_number >= 65) {
    search_result = true;
    *ret = serial_rom64;
  }

  if (!search_result || !serial_rom[0]) search_result = false;

  benchmark ("search_inner_maxim end");
  return true;
}
#endif

bool search_init (const struct drup_state * const state, const struct bus * const bus, bool * const presence) {
  try (ds2482_bus_search_init, (state->master, bus, presence))
  return true;
}

bool search_start (const struct drup_state * const state, uint8_t const device_family, uint64_t * const device_id) {
  struct master * const master = state->master;
  try (ds2482_bus_search_start, (master))
  benchmark ("after bus init");

  try (search_inner, (master, device_family, device_id))
  benchmark ("after search inner");
  return true;
}

==> w1druppel-0.2.3.tgz: package/src/search.c <==
#include <stddef.h>

#include <alleycat.h>

#include "private/benchmark.h"
#include "private/ds2482.h"
#include "private/i2c.h"
// --- this is circular, seems ok.
#include "private/search.h"
#include "private/util.h"

#ifdef SEARCH_TYPE_W1DIRECT

static int determine_bit (int const desc_bit, int const bit) {
  int prevDeviceId = 0;
  if      (bit == desc_bit) return 1;
  else if (bit >  desc_bit) return 0;
  else return ((prevDeviceId >> bit) & 0x1);
}

static bool some_device_responded (uint8_t const triplet) {
  return (triplet & 0x03) != 0x03;
}

// --- e.g. for 0x33, the LSB is 00110011, and reversed it's {1, 1, 0, 0, 1, 1, 0, 0}.
static void get_family_reversed (uint8_t const device_family, bool * const family_reversed) {
  for (int i = 0; i < 8; i++) {
    int x = device_family & (1 << i);
    family_reversed[i] = (bool) x;
  }
}

/** `device_family` is an optional hex value to filter for a specific family, which speeds up the
 * search slightly. For DS1961 it's 0x33. Set to 0 to disable filtering.
 */

static bool search_inner (const struct master * const master, uint8_t const device_family, uint64_t * const ret) {
  uint64_t device_id = 0;
  int desc_bit = 64;
  int bit = -1;

  bool family_reversed[8];
  if (device_family) get_family_reversed (device_family, family_reversed);

  bool broke = false;
  // --- each bit ~ 3ms, all I/O
  while (++bit < 64) {
    benchmark ("inner loop, bit %d", bit);
    uint8_t triplet;
    uint8_t search_bit = determine_bit (desc_bit, bit);
    if (bit == 0) debug("search_bit %d\n", search_bit);
    try (ds2482_triplet, (master, search_bit, &triplet))
    benchmark ("inner loop, bit %d, after I/O", bit);

    if (!some_device_responded (triplet)) {
      if (bit != 0) debug ("Quitting at bit %d\n", bit);
      broke = true;
      break;
    }

    uint64_t bit_val = triplet >> 2;
    device_id |= bit_val << bit;

    if (device_family) {
      if (bit < 8 && family_reversed[bit] != (bool) bit_val) {
        debug ("bad read, bit = %d, expected = %d, bit_val = %d\n", bit, family_reversed[bit], bit_val);
        broke = true;
        break;
      }
    }
  }

  *ret = broke ? 0 : device_id;

  return true;
}
#endif

#ifdef SEARCH_TYPE_MAXIM
static bool search_inner (const struct master * const master, uint64_t * const ret) {
  benchmark ("search_inner_maxim start");
  *ret = 0;

  unsigned char serial_rom[8] = {0};
  uint64_t serial_rom64 = 0;

  bool search_direction;
  int rom_bit_number = 1;
  int rom_byte_number = 0;
  unsigned char rom_byte_mask = 1;
  int search_result = 0;

  do {
    bool id_bit;
    bool cmp_id_bit;
    try (ds2482_read_bit, (master, &id_bit))
    try (ds2482_read_bit, (master, &cmp_id_bit))

    if (id_bit && cmp_id_bit) break;
    if (id_bit != cmp_id_bit) search_direction = id_bit;
    else search_direction = false;

    if (search_direction) serial_rom[rom_byte_number] |= rom_byte_mask;
    else serial_rom[rom_byte_number] &= ~rom_byte_mask;

    try (ds2482_write_bit, (master, (search_direction)))

    rom_bit_number++;
    rom_byte_mask <<= 1;

    serial_rom64 |= ((uint64_t) serial_rom[rom_byte_number]) << (rom_byte_number * 8);

    if (rom_byte_mask == 0) {
      rom_byte_number++;
      rom_byte_mask = 1;
    }
  } while (rom_byte_number < 8);

  if (rom_bit_number >= 65) {
    search_result = true;
    *ret = serial_rom64;
  }

  if (!search_result || !serial_rom[0]) search_result = false;

  benchmark ("search_inner_maxim end");
  return true;
}
#endif

bool search_init (const struct drup_state * const state, const struct bus * const bus, uint8_t const cfg, bool * const presence) {
  try (ds2482_bus_search_init, (state->master, bus, cfg, presence))
  return true;
}

bool search_start (const struct drup_state * const state, uint8_t const device_family, uint64_t * const device_id) {
  struct master * const master = state->master;
  try (ds2482_bus_search_start, (master))
  benchmark ("after bus init");

  try (search_inner, (master, device_family, device_id))
  benchmark ("after search inner");
  return true;
}
